import re
import yaml
import nbformat
from nbformat.v4.nbbase import new_raw_cell

_header_re = re.compile(r"^---\s*$")
_empty_re = re.compile(r"^\s*$")
_jupyter_re = re.compile(r"^jupyter\s*:\s*$")
_leftspace_re = re.compile(r"^\s")


def _as_dict(metadata):
    if isinstance(metadata, nbformat.NotebookNode):
        return {k: _as_dict(metadata[k]) for k in metadata.keys()}
    return metadata


def encoding_and_executable(self, nb):
    lines = []
    metadata = nb.get('metadata', {})

    if self.ext != '.Rmd' and 'executable' in metadata:
        lines.append('#!' + metadata['executable'])
        del metadata['executable']

    if 'encoding' in metadata:
        lines.append(metadata['encoding'])
        del metadata['encoding']
    elif self.ext == '.py':
        for cell in nb.cells:
            try:
                cell.source.encode('ascii')
            except (UnicodeEncodeError, UnicodeDecodeError):
                lines.append(_utf8_header)
                break

    return lines


def metadata_and_cell_to_header(self, nb):
    '''
    Return the text header corresponding to a notebook, and remove the
    first cell of the notebook if it contained the header
    '''

    header = []
    skipline = True

    if len(nb.cells):
        c = nb.cells[0]
        if c.cell_type == 'raw':
            lines = c.source.strip('\n\t ').splitlines()
            if len(lines) >= 2 \
                    and _header_re.match(lines[0]) \
                    and _header_re.match(lines[-1]):
                header = lines[1:-1]
                skipline = not c.metadata.get('noskipline', False)
                nb.cells = nb.cells[1:]

    metadata = _as_dict(nb.get('metadata', {}))

    if len(metadata):
        header.extend(yaml.safe_dump({'jupyter': metadata},
                                     default_flow_style=False).splitlines())

    if len(header):
        header = ['---'] + header + ['---']

    header = self.markdown_escape(header)

    if len(header) and skipline:
        header += ['']

    return header


# https://www.python.org/dev/peps/pep-0263/
_encoding_re = re.compile('^[ \t\f]*#.*?coding[:=][ \t]*([-_.a-zA-Z0-9]+)')
_utf8_header = '# -*- coding: utf-8 -*-'


def header_to_metadata_and_cell(self, lines):
    '''
    Return the metadata, first cell of notebook, and next loc in text
    '''

    header = []
    jupyter = []
    injupyter = False
    ended = False
    metadata = {}
    start = 0

    for i, line in enumerate(lines):
        if not line.startswith(self.prefix):
            if i == 0 and line.startswith('#!'):
                metadata['executable'] = line[2:]
                start = i + 1
                continue
            if i == 0 or (i == 1 and not _encoding_re.match(lines[0])):
                encoding = _encoding_re.match(line)
                if encoding:
                    if encoding.group(1) != 'utf-8':
                        raise ValueError('Encodings other than utf-8 '
                                         'are not supported')
                    if line != _utf8_header:
                        metadata['encoding'] = line
                    start = i + 1
                    continue
            break

        line = self.markdown_unescape(line)

        if i == start:
            if _header_re.match(line):
                continue
            else:
                break

        if i > start and _header_re.match(line):
            ended = True
            break

        if _jupyter_re.match(line):
            injupyter = True
        elif not _leftspace_re.match(line):
            injupyter = False

        if injupyter:
            jupyter.append(line)
        else:
            header.append(line)

    if ended:
        if len(jupyter):
            metadata.update(yaml.load('\n'.join(jupyter))['jupyter'])

        skipline = True
        if len(lines) > i + 1:
            line = self.markdown_unescape(lines[i + 1])
            if not _empty_re.match(line):
                skipline = False
            else:
                i = i + 1
        else:
            skipline = False

        if len(header):
            cell = new_raw_cell(source='\n'.join(['---'] + header + ['---']),
                                metadata={} if skipline else
                                {'noskipline': True})
        else:
            cell = None

        return metadata, cell, i + 1

    return metadata, None, start
